@@ -41,7 +41,6 @@ from firewall.server.config_zone import FirewallDConfigZone
 from firewall.server.config_ipset import FirewallDConfigIPSet
 from firewall.server.config_helper import FirewallDConfigHelper
 from firewall.core.io.zone import Zone
-from firewall.core.io.service import Service
 from firewall.core.io.icmptype import IcmpType
 from firewall.core.io.ipset import IPSet
 from firewall.core.io.helper import Helper
@@ -105,6 +104,9 @@ class FirewallDConfig(slip.dbus.service.Object):
                                                 "IndividualCalls": "readwrite",
                                                 "LogDenied": "readwrite",
                                                 "AutomaticHelpers": "readwrite",
+                                                "FirewallBackend": "readwrite",
+                                                "FlushAllOnReload": "readwrite",
+                                                "RFC3964_IPv4": "readwrite",
                                               })
 
     @handle_exceptions
@@ -484,7 +486,8 @@ class FirewallDConfig(slip.dbus.service.Object):
     def _get_property(self, prop):
         if prop not in [ "DefaultZone", "MinimalMark", "CleanupOnExit",
                          "Lockdown", "IPv6_rpfilter", "IndividualCalls",
-                         "LogDenied", "AutomaticHelpers" ]:
+                         "LogDenied", "AutomaticHelpers", "FirewallBackend",
+                         "FlushAllOnReload", "RFC3964_IPv4" ]:
             raise dbus.exceptions.DBusException(
                 "org.freedesktop.DBus.Error.InvalidArgs: "
                 "Property '%s' does not exist" % prop)
@@ -523,7 +526,19 @@ class FirewallDConfig(slip.dbus.service.Object):
             return dbus.String(value)
         elif prop == "AutomaticHelpers":
             if value is None:
-                value = config.FALLBACK_LOG_DENIED
+                value = config.FALLBACK_AUTOMATIC_HELPERS
+            return dbus.String(value)
+        elif prop == "FirewallBackend":
+            if value is None:
+                value = config.FALLBACK_FIREWALL_BACKEND
+            return dbus.String(value)
+        elif prop == "FlushAllOnReload":
+            if value is None:
+                value = "yes" if config.FALLBACK_FLUSH_ALL_ON_RELOAD else "no"
+            return dbus.String(value)
+        elif prop == "RFC3964_IPv4":
+            if value is None:
+                value = "yes" if config.FALLBACK_RFC3964_IPV4 else "no"
             return dbus.String(value)
 
     @dbus_handle_exceptions
@@ -544,6 +559,12 @@ class FirewallDConfig(slip.dbus.service.Object):
             return dbus.String(self._get_property(prop))
         elif prop == "AutomaticHelpers":
             return dbus.String(self._get_property(prop))
+        elif prop == "FirewallBackend":
+            return dbus.String(self._get_property(prop))
+        elif prop == "FlushAllOnReload":
+            return dbus.String(self._get_property(prop))
+        elif prop == "RFC3964_IPv4":
+            return dbus.String(self._get_property(prop))
         else:
             raise dbus.exceptions.DBusException(
                 "org.freedesktop.DBus.Error.InvalidArgs: "
@@ -583,7 +604,8 @@ class FirewallDConfig(slip.dbus.service.Object):
         if interface_name == config.dbus.DBUS_INTERFACE_CONFIG:
             for x in [ "DefaultZone", "MinimalMark", "CleanupOnExit",
                        "Lockdown", "IPv6_rpfilter", "IndividualCalls",
-                       "LogDenied", "AutomaticHelpers" ]:
+                       "LogDenied", "AutomaticHelpers", "FirewallBackend",
+                       "FlushAllOnReload", "RFC3964_IPv4" ]:
                 ret[x] = self._get_property(x)
         elif interface_name in [ config.dbus.DBUS_INTERFACE_CONFIG_DIRECT,
                                  config.dbus.DBUS_INTERFACE_CONFIG_POLICIES ]:
@@ -609,7 +631,9 @@ class FirewallDConfig(slip.dbus.service.Object):
         if interface_name == config.dbus.DBUS_INTERFACE_CONFIG:
             if property_name in [ "MinimalMark", "CleanupOnExit", "Lockdown",
                                   "IPv6_rpfilter", "IndividualCalls",
-                                  "LogDenied", "AutomaticHelpers" ]:
+                                  "LogDenied", "AutomaticHelpers",
+                                  "FirewallBackend", "FlushAllOnReload",
+                                  "RFC3964_IPv4" ]:
                 if property_name == "MinimalMark":
                     try:
                         int(new_value)
@@ -638,6 +662,22 @@ class FirewallDConfig(slip.dbus.service.Object):
                         raise FirewallError(errors.INVALID_VALUE,
                                             "'%s' for %s" % \
                                             (new_value, property_name))
+                if property_name == "FirewallBackend":
+                    if new_value not in config.FIREWALL_BACKEND_VALUES:
+                        raise FirewallError(errors.INVALID_VALUE,
+                                            "'%s' for %s" % \
+                                            (new_value, property_name))
+                if property_name == "FlushAllOnReload":
+                    if new_value.lower() not in ["yes", "true", "no", "false"]:
+                        raise FirewallError(errors.INVALID_VALUE,
+                                            "'%s' for %s" % \
+                                            (new_value, property_name))
+                if property_name == "RFC3964_IPv4":
+                    if new_value.lower() not in ["yes", "true", "no", "false"]:
+                        raise FirewallError(errors.INVALID_VALUE,
+                                            "'%s' for %s" % \
+                                            (new_value, property_name))
+
                 self.config.get_firewalld_conf().set(property_name, new_value)
                 self.config.get_firewalld_conf().write()
                 self.PropertiesChanged(interface_name,
@@ -1024,7 +1064,7 @@ class FirewallDConfig(slip.dbus.service.Object):
         raise FirewallError(errors.INVALID_SERVICE, service)
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG,
-                         in_signature='s'+Service.DBUS_SIGNATURE,
+                         in_signature='s(sssa(ss)asa{ss}asa(ss))',
                          out_signature='o')
     @dbus_handle_exceptions
     def addService(self, service, settings, sender=None):
@@ -1038,6 +1078,21 @@ class FirewallDConfig(slip.dbus.service.Object):
         config_service = self._addService(obj)
         return config_service
 
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG,
+                         in_signature='sa{sv}',
+                         out_signature='o')
+    @dbus_handle_exceptions
+    def addService2(self, service, settings, sender=None):
+        """add service with given name and settings
+        """
+        service = dbus_to_python(service, str)
+        settings = dbus_to_python(settings)
+        log.debug1("config.addService2('%s')", service)
+        self.accessCheck(sender)
+        obj = self.config.new_service_dict(service, settings)
+        config_service = self._addService(obj)
+        return config_service
+
     @dbus.service.signal(config.dbus.DBUS_INTERFACE_CONFIG, signature='s')
     @dbus_handle_exceptions
     def ServiceAdded(self, service):
